#############################
###                       ###
###     Jarbin-ToolKit    ###
###        config         ###
###   ----config.py----   ###
###                       ###
###=======================###
### by JARJARBIN's STUDIO ###
#############################


from builtins import object, type
from typing import Any


class Config:
    """
        Config class.

        Config file tool.
    """


    def __init__(
            self,
            path : str,
            data : dict | None = None,
            *,
            file_name : str = "config.ini"
        ) -> None:
        """
            Create a new config file if 'path'/'file_name' does not exist, read otherwise.

            Parameters:
                path (str): path to folder which you want your config file to be in
                data (dict | None, optional): data to put in the config file
                file_name (str, optional): name of config file
        """

        from configparser import ConfigParser
        from platform import system

        if path[-1] != "/":
            path += "/"

        self.config : ConfigParser | None = ConfigParser()
        self.path : str | None = path
        self.file_name : str | None = file_name

        if system() == "Windows":

            ## cannot be tested with pytest ##

            self.path = self.path.replace("/", "\\") # pragma: no cover
            self.file_name = self.file_name.replace("/", "\\") # pragma: no cover

        elif system() == "Linux":

            ## cannot be tested with pytest ##

            self.path = self.path.replace("\\", "/") # pragma: no cover
            self.file_name = self.file_name.replace("/", "\\") # pragma: no cover

        if Config.exist(self.path):
            self.config.read(self.path + self.file_name)

        else:
            if not data and self.file_name == "config.ini":
                data = {}

            try:
                open(str(self.path) + str(self.file_name), 'x').close()

            except FileExistsError:
                pass

            with open(str(self.path) + str(self.file_name), 'w') as config_file:
                for key in data:
                    self.config[key] = data[key]

                self.config.write(config_file)

            config_file.close()


    def set(
            self,
            section : str,
            option : str,
            data : Any
        ) -> None:
        """
            Set a new value in a config file.

            Parameters:
                section (str): section name
                option (str): option name
                data (Any): data to put in the config file
        """

        self.config.set(section, option, str(data))

        with open(str(self.path) + str(self.file_name), 'w') as config_file:
            self.config.write(config_file)

        config_file.close()


    def get(
            self,
            section : str,
            option : str,
            wanted_type : type = str
        ) -> Any:
        """
            Get a value from the config file.

            Parameters:
                section (str): section name
                option (str): option name
                wanted_type (type, optional): returned type

            Returns:
                Any: data retrieved from config file and of type 'wanted_type'
        """

        return wanted_type(self.config.get(section, option))


    def get_bool(
            self,
            section : str,
            option : str
        ) -> bool:
        """
            Get a value as a bool from the config file.

            Parameters:
                section (str): section name
                option (str): option name

            Returns:
                Any: data retrieved from config file and of type 'wanted_type'
        """

        return self.config.getboolean(section, option)


    def get_int(
            self,
            section : str,
            option : str
        ) -> Any:
        """
            Get a value as a int from the config file.

            Parameters:
                section (str): section name
                option (str): option name

            Returns:
                Any: data retrieved from config file and of type 'wanted_type'
        """

        return self.config.getint(section, option)


    def get_float(
            self,
            section : str,
            option : str
        ) -> Any:
        """
            Get a value as a float from the config file.

            Parameters:
                section (str): section name
                option (str): option name

            Returns:
                Any: data retrieved from config file and of type 'wanted_type'
        """

        return self.config.getfloat(section, option)


    def delete(
            self,
            cached : bool = False
        ) -> bool:
        """
            Delete the config file.

            Parameters:
                cached (bool, optional): keep the config file's data in memory

            Returns:
                bool: True if deleted else False
        """

        from os import remove

        remove(self.path + self.file_name)

        if not cached:
            self.config = None

        if not Config.exist(self.path, file_name=self.file_name):
            self.path = None
            self.file_name = None
            return True

        ## cannot be tested with pytest ##

        else:
            return False # pragma: no cover


    @staticmethod
    def exist(
            path : str,
            *,
            file_name : str = "config.ini"
        ) -> bool:
        """
            Check if a config file config.ini is empty or doesn't exist

            Returns:
                bool: False if empty or not existing, True otherwise
                file_name (str, optional): name of config file
        """

        if path[-1] != "/":
            path += "/"

        empty_config : bool = True

        try :
            with open(path + file_name, 'r') as config_file:
                if config_file.read() == "":
                    empty_config = False
            config_file.close()

        except FileNotFoundError:
            empty_config = False

        return empty_config


    def __repr__(
            self
        ) -> str:
        """
            Convert Config object to string.

            Returns:
                str: Config string
        """

        return f"Config({repr(self.path)}, ?, {repr(self.file_name)})"



import pytest


from jarbin_toolkit_config import Config


def test_config_exist_returns_false(
    ) -> None:
    assert not Config.exist("tests")


def test_config_create_and_dont_exist(
    ) -> None:
    data = {
        "SECTION1": {"key11": "value11", "key12": "value12"},
        "SECTION2": {"key21": True, "key22": False},
        "SECTION3": {"key31": -100, "key32": 0, "key33": 100},
        "SECTION4": {"key41": -0.5, "key42": 0, "key43": 0.5}
    }
    Config("tests", data)

    assert Config.exist("tests")


def test_config_create_and_exist(
    ) -> None:
    Config("tests")

    assert Config.exist("tests")


def test_config_write(
    ) -> None:
    config = Config("tests")

    assert config.get("SECTION1", "key11") == "value11"
    config.set("SECTION1", "key11", "value1111")
    assert config.get("SECTION1", "key11") == "value1111"

    assert Config.exist("tests")


def test_config_read(
    ) -> None:
    config = Config("tests")
    assert config.get("SECTION1", "key11") == "value1111"
    assert config.get("SECTION1", "key12") == "value12"
    assert config.get_bool("SECTION2", "key21") == True
    assert config.get_bool("SECTION2", "key22") == False
    assert config.get_int("SECTION3", "key31") == -100
    assert config.get_int("SECTION3", "key32") == 0
    assert config.get_int("SECTION3", "key33") == 100
    assert config.get_float("SECTION4", "key41") == -0.5
    assert config.get_float("SECTION4", "key42") == 0
    assert config.get_float("SECTION4", "key43") == 0.5


def test_config_delete_cached(
    ) -> None:
    config = Config("tests")
    assert config.delete(True)
    assert config.config
    assert not Config.exist("tests")


def test_config_repr(
    ) -> None:
    config = Config("tests")

    assert repr(config) == "Config(\'tests/\', ?, \'config.ini\')"


def test_config_delete_not_cached(
    ) -> None:
    config = Config("tests")
    assert config.delete()
    assert not config.config
    assert not Config.exist("tests")


def test_config_delete_not_exist(
    ) -> None:
    assert not Config.exist("tests")



#!/usr/bin/env python3
"""
    FULL Jarbin-ToolKit:Config – Config MODULE DEMO
    ========================================

    This script is a REAL MANUAL TEST of the Config module.

    It covers:
    - Action
    - Actions (single, list, iteration, execution)

    Run in a REAL terminal.
"""


def config_demo(
    ) -> None:
    from jarbin_toolkit_config import Config


    # ============================================================
    # ACTION – BASIC
    # ============================================================

    print("\n=== ACTION BASIC ===")

    def say_hello(name):
        print(f"Hello {name}")

    action = Action("hello_action", say_hello, "Epitech")

    action.function(*action.args, **action.kwargs)

    print("\n(Action executed manually)")


    # ============================================================
    # ACTION – WITH KEYWORDS
    # ============================================================

    print("\n=== ACTION WITH KWARGS ===")

    def show_data(a, b, c=0):
        print(f"a={a}, b={b}, c={c}")

    action = Action("data_action", show_data, 1, 2, c=3)
    action.function(*action.args, **action.kwargs)

    print("\n(Action with kwargs executed)")


    # ============================================================
    # ACTIONS – SINGLE ACTION
    # ============================================================

    print("\n=== ACTIONS SINGLE ===")

    actions = Actions(action)

    for act in actions.actions:
        act.function(*act.args, **act.kwargs)

    print("\n(Actions container with one Action works)")


    # ============================================================
    # ACTIONS – MULTIPLE ACTIONS
    # ============================================================

    print("\n=== ACTIONS MULTIPLE ===")

    a1 = Action("a1", print, "First action")
    a2 = Action("a2", print, "Second action")
    a3 = Action("a3", print, "Third action")

    actions = Actions([a1, a2, a3])

    for act in actions.actions:
        act.function(*act.args, **act.kwargs)

    print("\n(All actions executed in order)")


    # ============================================================
    # FINAL MESSAGE
    # ============================================================

    print("\n=== CONFIG MODULE DEMO COMPLETE ===")
    print("If all outputs behaved as described, the System module works as expected.")


if __name__ == "__main__":
    config_demo()
